/*
 * Copyright 2019-2019 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.cloud.function.context.catalog;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import org.junit.Test;
import reactor.core.publisher.Flux;

import org.springframework.messaging.support.MessageBuilder;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Dave Syer
 */
public class MessageConsumerTests {

	private List<String> items = new ArrayList<>();

	@Test
	public void plainConsumer() {
		MessageConsumer consumer = new MessageConsumer(input());
		consumer.accept(Flux
				.just(MessageBuilder.withPayload("foo").setHeader("foo", "bar").build()));
		assertThat(this.items).hasSize(1);
	}

	private Consumer<String> input() {
		return value -> this.items.add(value);
	}

}
